"""Help the user download and install the missing Ragdoll SDK"""

import bpy
import webbrowser


class VisitDownloadPage(bpy.types.Operator):
    bl_idname = "ragdoll.visit_download_page"
    bl_label = "Download Ragdoll Core"
    bl_options = {"REGISTER"}
    bl_description = (
        "Click here to visit the download page for Ragdoll Core"
    )

    def execute(self, context):
        url = "https://learn.ragdolldynamics.com/download"

        # Help those looking at the terminal
        print("Go to %s" % url)

        # Help those looking at their UI with a browser available
        webbrowser.open_new(url)

        return {"FINISHED"}


class UnsupportedPlatformLinux(bpy.types.Operator):
    bl_idname = "ragdoll.unsupported_platform_linux"
    bl_label = "Coming soon for Linux"
    bl_options = {"REGISTER"}
    bl_description = "Patience is key"

    def execute(self, context):
        self.report({"INFO"}, "Soon")
        return {"FINISHED"}


class UnsupportedPlatformMacOS(bpy.types.Operator):
    bl_idname = "ragdoll.unsupported_platform_macos"
    bl_label = "Coming soon for MacOS"
    bl_options = {"REGISTER"}
    bl_description = "Patience is key"

    def execute(self, context):
        self.report({"INFO"}, "Soon")
        return {"FINISHED"}


class PlaceholderMenu(bpy.types.Menu):
    bl_label = "Ragdoll"
    bl_idname = "ANIMATION_MT_ragdoll_menu"

    def draw(self, _context):
        layout = self.layout
        col = layout.column()

        for item in _menu_items:
            col.operator(item, icon="EXPORT")


_menu_items = []


def _draw_menu(self, _context):
    self.layout.menu(PlaceholderMenu.bl_idname)


def install(platform, core_version):
    if platform == "Windows":
        VisitDownloadPage.bl_label += " %s" % core_version
        _menu_items.append(VisitDownloadPage.bl_idname)
        bpy.utils.register_class(VisitDownloadPage)

    if platform == "Darwin":
        _menu_items.append(UnsupportedPlatformMacOS.bl_idname)
        bpy.utils.register_class(UnsupportedPlatformMacOS)

    if platform == "Linux":
        _menu_items.append(UnsupportedPlatformLinux.bl_idname)
        bpy.utils.register_class(UnsupportedPlatformLinux)

    bpy.utils.register_class(PlaceholderMenu)

    bpy.types.VIEW3D_MT_editor_menus.append(_draw_menu)


def uninstall(platform, core_version):
    if platform == "Windows":
        bpy.utils.unregister_class(VisitDownloadPage)

    if platform == "Darwin":
        bpy.utils.unregister_class(UnsupportedPlatformMacOS)

    if platform == "Linux":
        bpy.utils.unregister_class(UnsupportedPlatformLinux)

    bpy.utils.unregister_class(PlaceholderMenu)

    bpy.types.VIEW3D_MT_editor_menus.remove(_draw_menu)
    _menu_items[:] = []
